# coding: utf-8
from datetime import datetime
from dateutil.relativedelta import relativedelta
from decorators import check_dul_type
from random import randint
from random import sample
from names import AdaptationProgram
from names import COLLEGE_SUBJECTS
from names import DifficultSituation
from names import DisabilityGroup
from names import HealthGroups
from names import KINDERGARTEN_NAMES
from names import LongTermTreatment
from names import PhysicalCultureGroup
from names import SCHOOL_SUBJECTS


def get_id():
    return datetime.now().strftime('%y%m%d%H%M%S%f')


class Organization(object):
    u"""
    Класс для формирования данных по организации.
    """
    def __init__(self, region, city, oktmo, system, factory):
        self.id = get_id()
        self.system = system
        self._factory = factory
        self.name = self._get_name()
        self.region = region
        self.city = city
        self.oktmo = oktmo
        self.inn = self._get_inn()
        self.kpp = randint(9000000, 9999999)
        self.ogrn = self._get_ogrn()
        self.other_id = get_id()
        self.prof_id = get_id()
        self.act_cap = randint(500, 750)
        self.max_cap = randint(750, 1000)

    @staticmethod
    def _get_inn():
        factors = [2, 4, 10, 3, 5, 9, 4, 6, 8]
        numbers = [randint(1, 9) for _ in xrange(9)]
        str_num = [str(n) for n in numbers]
        numbers = sum([factors[i]*numbers[i] for i in xrange(len(numbers))])
        check_sum = str(numbers % 11) if numbers % 11 != 10 else '0'

        return ''.join([''.join(str_num), check_sum])

    @staticmethod
    def _get_ogrn():
        num = randint(9100000000, 9999999999)
        return ''.join([str((num*11)+3), '3'])

    def _get_name(self):
        if self.system.lower() == 'dou':
            return KINDERGARTEN_NAMES[randint(0, len(KINDERGARTEN_NAMES)-1)]
        else:
            return self._factory.company().split(' ')[0].replace(',', '')


def mult(inn, factors):
    return [int(inn[i])*factors[i] for i in xrange(len(factors))]


class Person(object):
    u"""
    Класс для формирования данных по персоне.
    """
    def __init__(
            self, firstname, middlename, lastname,
            delegate_firstname, delegate_middlename,
            delegate_lastname, organization, system,
            is_all, need_health, the_last_system, dul_gen_type
    ):
        self.id = get_id()
        self.is_all = is_all
        self.system = system
        self.dul_gen_type = dul_gen_type
        self.the_last_system = the_last_system
        self.need_health = need_health
        self.firstname = firstname
        self.middlename = middlename
        self.lastname = lastname
        self.snils = self._get_snils(is_child=True)
        self.document_type, self.document_id, self.document_series, \
            self.document_number = self._get_document(is_child=True)
        self.decl_id = get_id()
        self.decl_org_id = get_id()
        self.other_id = get_id()
        self.delegate_id = get_id()
        self.delegate_firstname = delegate_firstname
        self.delegate_middlename = delegate_middlename
        self.delegate_lastname = delegate_lastname
        self.delegate_snils = self._get_snils()
        self.delegate_doc_type, \
            self.delegate_doc_id, \
            self.delegate_doc_series, \
            self.delegate_doc_number = self._get_document(is_child=False)
        self.delegate_person_id = get_id()
        self.organization = organization
        self.edu_id = get_id()
        self.project_id = get_id()
        self.org_activities_id = get_id()
        self.activities_id = get_id()
        self.achieve_id = get_id()
        self.ex_activities_id = get_id()
        self.self_stud_id = get_id()
        self.main_edu_class = get_id()
        self.basic_edu_mark_id = get_id()
        self.common_edu_mark_id = get_id()
        self.mid_edu_id = get_id()
        self.ann_mark_id = get_id()
        self.final_mark_id = get_id()
        self.employment_id = get_id()
        self.prof_edu_id = get_id()
        self.addit_edu_id = get_id()
        self.addit_edu_score_id = get_id()
        self.health_group = self._get_random_value(HealthGroups)  # HealthGroup
        self.long_term = self._get_random_value(LongTermTreatment)  # LongTermTreatment
        self.dis_group = self._get_random_value(DisabilityGroup)  # DisabilityGroup
        self.dis_date = '{year}-{month:02d}-{day:02d}'.format(
            year=randint(2016, 2017),
            month=randint(1, 12),
            day=randint(1, 28)
        )
        self.difficult_situation = self._get_random_value(DifficultSituation)  # DifficultSituation
        self.adapt_program = self._get_random_value(AdaptationProgram)  # AdaptationProgram
        self.phys_cult_group = self._get_random_value(PhysicalCultureGroup)  # PhysicalCultureGroup
        self._dob = self._create_dob(is_child=True)
        self.dob = self._get_date_of_birth(is_child=True)
        self._delegate_dob = self._create_dob(is_child=False)
        self.delegate_dob = self._get_date_of_birth(is_child=False)
        self.document_date = '{year}-{month:02d}-{day:02d}'.format(
            year=2015,
            month=randint(1, 12),
            day=randint(1, 28)
        )
        self.basic_edu_subject = self._get_subject()
        self.basic_score = randint(60, 100)
        self.basic_mark = self._get_mark(score=self.basic_score)
        self.common_edu_subject = self._get_subject()
        self.common_score = randint(60, 100)
        self.common_mark = self._get_mark(score=self.common_score)
        self.enroll_act_number = randint(100, 999)
        self.deduct_act_number = randint(100000, 999999)
        self.certificate_series = randint(1000, 9999)
        self.certificate_number = randint(100000, 999999)
        # Для ЭДС
        self.dou_declaration_date = (
            self._dob + relativedelta(months=1)).strftime('%Y-%m-%d')
        self.dou_enroll_date = '{year}-09-01'.format(year=self._dob.year+2)
        self.dou_deduct_date = ('{year}-08-15'.format(year=self._dob.year+6)
                                if is_all or not self.the_last_system
                                else None)
        # Для ЭШ
        self.school_declaration_date = '{year}-08-15'.format(
            year=self._dob.year + 14
        )
        self.school_enroll_date = '{year}-09-01'.format(
            year=self._dob.year + 14
        )
        self.school_deduct_date = (
            '{year}-08-31'.format(year=self._dob.year + 15)
            if is_all or not self.the_last_system else None
        )
        # Для ЭДО
        self.extedu_declaration_date = '{year}-09-01'.format(
            year=self._dob.year + 8
        )
        self.extedu_enroll_date = '{year}-09-01'.format(
            year=self._dob.year + 8
        )
        # Для ЭК
        self.ssuz_declaration_date = '{year}-08-15'.format(
            year=self._dob.year + 15
        )
        self.ssuz_enroll_date = '{year}-09-01'.format(
            year=self._dob.year + 15
        )
        self.ssuz_deduct_date = (
            '{year}-08-31'.format(year=self._dob.year + 16)
            if is_all or not self.the_last_system else None
        )
        # для ЭПО
        self.epo_declaration_date = '{year}-09-01'.format(
            year=self._dob.year + 16
        )
        self.epo_enroll_date = '{year}-09-01'.format(year=self._dob.year + 16)

    def _create_dob(self, is_child):

        if is_child:
            if self.is_all:
                dob_year = randint(1995, 1996)
            else:
                dob_year = 2000
        else:
            dob_year = randint(1970, 1975)
        return datetime(dob_year, randint(1, 12), randint(1, 28))

    def _get_date_of_birth(self, is_child):
        if is_child:
            return self._dob.strftime('%Y-%m-%d')
        else:
            return self._delegate_dob.strftime('%Y-%m-%d')

    @check_dul_type
    def _get_snils(self, is_child=False):

        def get_check_sum(current_sum):
            check = current_sum
            while check > 100:
                check %= 101
            else:
                if check == 100:
                    check = 0
            return check

        numbers = sample(xrange(10), 9)

        str_num = [str(n) for n in numbers]
        multipls = list(xrange(1, 10))
        multipls.reverse()
        numbers = [numbers[i] * multipls[i] for i in xrange(len(numbers))]
        num_sum = sum(numbers)
        check_sum = '%02d' % (get_check_sum(num_sum))

        for i in (3, 7):
            str_num.insert(i, '-')

        return ' '.join([''.join(str_num), check_sum])

    @check_dul_type
    def _get_document(self, is_child=True):
        doc_id = get_id()
        SERIES_DICT = {
            1: u'V-ТФ',
            2: u'V-АГ'
        }
        number = '%06d' % randint(0, 999999)
        if is_child and (self.system == 'dou' or self.system == 'school'):
            series = SERIES_DICT[randint(1, 2)]
            doc_type = u'Свидетельство о рождении'
        else:
            doc_type = u'Паспорт гражданина РФ'
            series = '%04d' % randint(0, 9999)
        return doc_type, doc_id, series, number

    @staticmethod
    def _get_random_value(some_dict):
        keys = list(some_dict.iterkeys())
        keys_len = len(keys)
        return some_dict[keys[randint(0, keys_len-1)]]

    def _get_subject(self):
        if self.system == 'school':
            subj_list = SCHOOL_SUBJECTS
        else:
            subj_list = COLLEGE_SUBJECTS

        subj_len = len(subj_list)-1
        return subj_list[randint(0, subj_len)]

    def _get_mark(self, score):
        if self.system == 'college':
            return randint(3, 5)
        else:
            if 60 <= score < 75:
                mark = 3
            elif 75 <= score < 90:
                mark = 4
            else:
                mark = 5

        return mark
